// UBC CPSC 314 (2015W2) -- P1
// HAVE FUN!!! :)

// ASSIGNMENT-SPECIFIC API EXTENSION
THREE.Object3D.prototype.setMatrix = function(a) {
  this.matrix=a;
  this.matrix.decompose(this.position,this.quaternion,this.scale);
}

// SETUP RENDERER & SCENE
var canvas = document.getElementById('canvas');
var scene = new THREE.Scene();
var renderer = new THREE.WebGLRenderer();
renderer.setClearColor(0xFFFFFF); // white background colour
canvas.appendChild(renderer.domElement);

// SETUP CAMERA
var camera = new THREE.PerspectiveCamera(30,1,0.1,1000); // view angle, aspect ratio, near, far
camera.position.set(45,20,40);
camera.lookAt(scene.position);
scene.add(camera);

// SETUP ORBIT CONTROLS OF THE CAMERA
var controls = new THREE.OrbitControls(camera);

// ADAPT TO WINDOW RESIZE
function resize() {
  renderer.setSize(window.innerWidth,window.innerHeight);
  camera.aspect = window.innerWidth/window.innerHeight;
  camera.updateProjectionMatrix();
}

// EVENT LISTENER RESIZE
window.addEventListener('resize',resize);
resize();

//SCROLLBAR FUNCTION DISABLE
window.onscroll = function () {
     window.scrollTo(0,0);
   }

// SETUP HELPER GRID
// Note: Press Z to show/hide
var gridGeometry = new THREE.Geometry();
var i;
for(i=-50;i<51;i+=2) {
    gridGeometry.vertices.push( new THREE.Vector3(i,0,-50));
    gridGeometry.vertices.push( new THREE.Vector3(i,0,50));
    gridGeometry.vertices.push( new THREE.Vector3(-50,0,i));
    gridGeometry.vertices.push( new THREE.Vector3(50,0,i));
}

var gridMaterial = new THREE.LineBasicMaterial({color:0xBBBBBB});
var grid = new THREE.Line(gridGeometry,gridMaterial,THREE.LinePieces);

/////////////////////////////////
//   YOUR WORK STARTS BELOW    //
/////////////////////////////////

// MATERIALS
// Note: Feel free to be creative with this! 

//////////////////////////////////////////////////////////////////////////////////
// ADD SOME LIGHTS
// Source: http://mrdoob.github.io/three.js/examples/webgl_lights_hemisphere.html
//////////////////////////////////////////////////////////////////////////////////
hemiLight = new THREE.HemisphereLight( 0xffffff, 0xffffff, 0.6 );
hemiLight.color.setHSL( 0.6, 1, 0.6 );
hemiLight.groundColor.setHSL( 0.095, 1, 0.75 );
hemiLight.position.set( 0, 500, 0 );
scene.add( hemiLight );

dirLight = new THREE.DirectionalLight( 0xffffff, 1 );
dirLight.color.setHSL( 0.1, 1, 0.95 );
dirLight.position.set( -1, 1.75, 1 );
dirLight.position.multiplyScalar( 50 );
scene.add( dirLight );
dirLight.castShadow = true;
dirLight.shadowMapWidth = 2048;
dirLight.shadowMapHeight = 2048;
var d = 50;
dirLight.shadowCameraLeft = -d;
dirLight.shadowCameraRight = d;
dirLight.shadowCameraTop = d;
dirLight.shadowCameraBottom = -d;
dirLight.shadowCameraFar = 3500;
dirLight.shadowBias = -0.0001;

var light = new THREE.PointLight(0xffffff, 0.2);
light.position.set(0, 50, 20);
scene.add(light);

//////////////////////////////////////////////////////////////////////////////////
// Try different materials
//////////////////////////////////////////////////////////////////////////////////
red = new THREE.Color("rgb(255, 50, 50)")
redder = new THREE.Color("rgb(170, 10, 10)")
redderer = new THREE.Color("rgb(255, 120, 120)")
pinkish = new THREE.Color("rgb(255, 120, 120)")
greyish = new THREE.Color("rgb(100, 100, 100)")
darkgreyish = new THREE.Color("rgb(80, 80, 80)")
orange = new THREE.Color("rgb(240,128,128)")

var lambertMaterial = new THREE.MeshLambertMaterial({
  color: greyish,
  shininess: 30
});

var lambertHeadMaterial = new THREE.MeshLambertMaterial({
  color: darkgreyish,
  shininess: 30
});

var noseMaterial = new THREE.MeshLambertMaterial({
  color: red,
  shininess: 30
});

var tentacleMaterial = new THREE.MeshLambertMaterial({
  color: redder,
  shininess: 30
});

var smallTentacleMaterial = new THREE.MeshLambertMaterial({
  color: redderer,
  shininess: 25
});

var pawMaterial = new THREE.MeshLambertMaterial({
  color: pinkish,
  shininess: 20
});

var clawMaterial = new THREE.MeshLambertMaterial({
  color: red,
  shininess: 10
});

var fireballMaterial = new THREE.MeshLambertMaterial({
  color: orange,
  shininess: 5
});

var normalMaterial = new THREE.MeshNormalMaterial();

// function drawCube()
// Draws a unit cube centered about the origin.
// Note: You will be using this for all of your geometry
function makeCube() {
  var unitCube = new THREE.BoxGeometry(1,1,1);
  return unitCube;
}

//////////////////////////////////////////////////////////////////////////////////
// Make a Floor!
// Grass: http://svg-wow.org/res/2.0/images/textures/grass.jpg
//////////////////////////////////////////////////////////////////////////////////
var floorTexture = new THREE.ImageUtils.loadTexture( 'images/cloud.jpg' );
floorTexture.wrapS = floorTexture.wrapT = THREE.RepeatWrapping; 
floorTexture.repeat.set( 1, 1 );
var floorMaterial = new THREE.MeshBasicMaterial( { map: floorTexture, side: THREE.DoubleSide } );
var floorGeometry = new THREE.PlaneGeometry(700, 700, 10, 10);
floorMaterial.transparent = true;
floorMaterial.opacity = 0.45;
var floor = new THREE.Mesh(floorGeometry, floorMaterial);
floor.position.y = -1;
floor.rotation.x = Math.PI / 2;
floor.receiveShadow = true;
scene.add(floor);

//////////////////////////////////////////////////////////////////////////////////
// Add a Sky Box!
// https://www.script-tutorials.com/webgl-with-three-js-lesson-5/
//////////////////////////////////////////////////////////////////////////////////
var path = 'images/skybox/';
// var sides = [ path + 'posx.jpg', path + 'negx.jpg', path + 'posy.jpg', path + 'negy.jpg', path + 'posz.jpg', path + 'negz.jpg' ];
var sides = [ path + 'sbox_px.jpg', path + 'sbox_nx.jpg', path + 'sbox_py.jpg', path + 'sbox_ny.jpg', path + 'sbox_pz.jpg', path + 'sbox_nz.jpg' ];
var scCube = THREE.ImageUtils.loadTextureCube(sides);
scCube.format = THREE.RGBFormat;
var skyShader = THREE.ShaderLib["cube"];
skyShader.uniforms["tCube"].value = scCube;
var skyMaterial = new THREE.ShaderMaterial( {
  fragmentShader: skyShader.fragmentShader, vertexShader: skyShader.vertexShader,
  uniforms: skyShader.uniforms, depthWrite: false, side: THREE.BackSide
});

var skyBox = new THREE.Mesh(new THREE.CubeGeometry(1000, 1000, 1000), skyMaterial);
skyMaterial.needsUpdate = true;
scene.add(skyBox);

//////////////////////////////////////////////////////////////////////////////////
// GEOMETRY OF THE MOLE!
//////////////////////////////////////////////////////////////////////////////////
var torsoY = 5
var torsoX = 5
var torsoZ = 8

var headZ = 4
var noseZ = 2
var tailZ = 5

var noseX = 3.5
var noseY = 2.0

var fireballGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1);
fireballGeometry.applyMatrix(non_uniform_scale);

var torsoGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(torsoX,0,0,0, 0,torsoY,0,0, 0,0,torsoZ,0, 0,0,0,1);
torsoGeometry.applyMatrix(non_uniform_scale);

var headGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(3.5,0,0,0, 0,4,0,0, 0,0,headZ,0, 0,0,0,1);
headGeometry.applyMatrix(non_uniform_scale);

var noseGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(noseX,0,0,0, 0,noseY,0,0, 0,0,noseZ,0, 0,0,0,1);
noseGeometry.applyMatrix(non_uniform_scale);

var smallTentacleGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(0.35,0,0,0, 0,1.5,0,0, 0,0,0.2,0, 0,0,0,1);
smallTentacleGeometry.applyMatrix(non_uniform_scale);

var largeTentacleGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(0.5,0,0,0, 0,3.5,0,0, 0,0,0.2,0, 0,0,0,1);
largeTentacleGeometry.applyMatrix(non_uniform_scale);

var pawX = 4
var pawY = 5
var pawZ = 1
var pawGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(pawX,0,0,0, 0,pawY,0,0, 0,0,pawZ,0, 0,0,0,1);
pawGeometry.applyMatrix(non_uniform_scale);

var clawGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(0.5,0,0,0, 0,1.5,0,0, 0,0,1,0, 0,0,0,1);
clawGeometry.applyMatrix(non_uniform_scale);

var tailGeometry = makeCube();
var non_uniform_scale = new THREE.Matrix4().set(0.5,0,0,0, 0,1.5,0,0, 0,0,tailZ,0, 0,0,0,1);
tailGeometry.applyMatrix(non_uniform_scale);

//////////////////////////////////////////////////////////////////////////////////
// TO-DO: SPECIFY THE REST OF YOUR STAR-NOSE MOLE'S GEOMETRY. 
// Note: You will be using transformation matrices to set the shape. 
// Note: You are not allowed to use the tools Three.js provides for 
//       rotation, translation and scaling.
// Note: The torso has been done for you (but feel free to modify it!)  
// Hint: Explicity declare new matrices using Matrix4().set     
//////////////////////////////////////////////////////////////////////////////////

// MATRICES
/////////////////////////////
// The Body!
/////////////////////////////
var torsoMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,2.5, 0,0,1,0, 0,0,0,1);
var torsoTrackMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1);

/////////////////////////////
// The Head
/////////////////////////////
var headMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,0, 0,0,1,(torsoZ+headZ)/2, 0,0,0,1);
var headTorsoMatrix = new THREE.Matrix4().multiplyMatrices(headMatrix, torsoMatrix);

/////////////////////////////
// The Nose
/////////////////////////////
var noseMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,0, 0,0,1,(headZ+noseZ)/2, 0,0,0,1);
var noseHeadMatrix = new THREE.Matrix4().multiplyMatrices(noseMatrix, headTorsoMatrix);

/////////////////////////////
// The Tail
/////////////////////////////
var tailMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,0, 0,0,1,-(torsoZ+tailZ)/2, 0,0,0,1);
var tailTorsoMatrix = new THREE.Matrix4().multiplyMatrices(tailMatrix, torsoMatrix);

/////////////////////////////
// The Front Paws
/////////////////////////////
var rotDegree =  -Math.PI/8
var z_out = (torsoZ + pawZ)/3
var x_out = (torsoX + pawX)/3.5
var y_out = (torsoX + pawX)/4

var frontLeftPawMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var fLTorsoMatrix = new THREE.Matrix4().multiplyMatrices(torsoMatrix, frontLeftPawMatrix);
var frontRightPawMatrix = new THREE.Matrix4().set(1,0,0,x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var fRTorsoMatrix = new THREE.Matrix4().multiplyMatrices(torsoMatrix, frontRightPawMatrix);

/////////////////////////////
// The Back Paws
/////////////////////////////
var backLeftPawMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),-z_out, 0,0,0,1);
var bLTorsoMatrix = new THREE.Matrix4().multiplyMatrices(torsoMatrix, backLeftPawMatrix);
var backRightPawMatrix = new THREE.Matrix4().set(1,0,0,x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),-z_out, 0,0,0,1);
var bRTorsoMatrix = new THREE.Matrix4().multiplyMatrices(torsoMatrix, backRightPawMatrix);

/////////////////////////////
// The 'fireballs'
/////////////////////////////
var fireballMatrix = new THREE.Matrix4().set(1,0,0,-2, 0,1,0,10, 0,0,1,2.5, 0,0,0,1);
var fireballMovementMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,1, 0,0,1,0, 0,0,0,1);

/////////////////////////////
// The Front Left Claws
/////////////////////////////
var clawY = pawY/2
var clawZ = 0

var fLClaw_1_Matrix = new THREE.Matrix4().set(1,0,0,-2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fLClaw_1_fLPawMatrix = new THREE.Matrix4().multiplyMatrices(fLTorsoMatrix, fLClaw_1_Matrix);
var fLClaw_2_Matrix = new THREE.Matrix4().set(1,0,0,-1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fLClaw_2_fLPawMatrix = new THREE.Matrix4().multiplyMatrices(fLTorsoMatrix, fLClaw_2_Matrix);
var fLClaw_3_Matrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fLClaw_3_fLPawMatrix = new THREE.Matrix4().multiplyMatrices(fLTorsoMatrix, fLClaw_3_Matrix);
var fLClaw_4_Matrix = new THREE.Matrix4().set(1,0,0,1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fLClaw_4_fLPawMatrix = new THREE.Matrix4().multiplyMatrices(fLTorsoMatrix, fLClaw_4_Matrix);
var fLClaw_5_Matrix = new THREE.Matrix4().set(1,0,0,2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fLClaw_5_fLPawMatrix = new THREE.Matrix4().multiplyMatrices(fLTorsoMatrix, fLClaw_5_Matrix);

/////////////////////////////
// The Front Right Claws
/////////////////////////////
var clawY = pawY/2
var clawZ = 0

var fRClaw_1_Matrix = new THREE.Matrix4().set(1,0,0,-2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fRClaw_1_fRPawMatrix = new THREE.Matrix4().multiplyMatrices(fRTorsoMatrix, fRClaw_1_Matrix);
var fRClaw_2_Matrix = new THREE.Matrix4().set(1,0,0,-1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fRClaw_2_fRPawMatrix = new THREE.Matrix4().multiplyMatrices(fRTorsoMatrix, fRClaw_2_Matrix);
var fRClaw_3_Matrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fRClaw_3_fRPawMatrix = new THREE.Matrix4().multiplyMatrices(fRTorsoMatrix, fRClaw_3_Matrix);
var fRClaw_4_Matrix = new THREE.Matrix4().set(1,0,0,1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fRClaw_4_fRPawMatrix = new THREE.Matrix4().multiplyMatrices(fRTorsoMatrix, fRClaw_4_Matrix);
var fRClaw_5_Matrix = new THREE.Matrix4().set(1,0,0,2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var fRClaw_5_fRPawMatrix = new THREE.Matrix4().multiplyMatrices(fRTorsoMatrix, fRClaw_5_Matrix);

/////////////////////////////
// The Back Left Claws
/////////////////////////////
var clawY = pawY/2
var clawZ = 0

var bLClaw_1_Matrix = new THREE.Matrix4().set(1,0,0,-2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bLClaw_1_bLPawMatrix = new THREE.Matrix4().multiplyMatrices(bLTorsoMatrix, bLClaw_1_Matrix);
var bLClaw_2_Matrix = new THREE.Matrix4().set(1,0,0,-1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bLClaw_2_bLPawMatrix = new THREE.Matrix4().multiplyMatrices(bLTorsoMatrix, bLClaw_2_Matrix);
var bLClaw_3_Matrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bLClaw_3_bLPawMatrix = new THREE.Matrix4().multiplyMatrices(bLTorsoMatrix, bLClaw_3_Matrix);
var bLClaw_4_Matrix = new THREE.Matrix4().set(1,0,0,1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bLClaw_4_bLPawMatrix = new THREE.Matrix4().multiplyMatrices(bLTorsoMatrix, bLClaw_4_Matrix);
var bLClaw_5_Matrix = new THREE.Matrix4().set(1,0,0,2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bLClaw_5_bLPawMatrix = new THREE.Matrix4().multiplyMatrices(bLTorsoMatrix, bLClaw_5_Matrix);


/////////////////////////////
// The Back Right Claws
/////////////////////////////
var clawY = pawY/2
var clawZ = 0

var bRClaw_1_Matrix = new THREE.Matrix4().set(1,0,0,-2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bRClaw_1_bRPawMatrix = new THREE.Matrix4().multiplyMatrices(bRTorsoMatrix, bRClaw_1_Matrix);
var bRClaw_2_Matrix = new THREE.Matrix4().set(1,0,0,-1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bRClaw_2_bRPawMatrix = new THREE.Matrix4().multiplyMatrices(bRTorsoMatrix, bRClaw_2_Matrix);
var bRClaw_3_Matrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bRClaw_3_bRPawMatrix = new THREE.Matrix4().multiplyMatrices(bRTorsoMatrix, bRClaw_3_Matrix);
var bRClaw_4_Matrix = new THREE.Matrix4().set(1,0,0,1.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bRClaw_4_bRPawMatrix = new THREE.Matrix4().multiplyMatrices(bRTorsoMatrix, bRClaw_4_Matrix);
var bRClaw_5_Matrix = new THREE.Matrix4().set(1,0,0,2.0, 0,1,0,-clawY, 0,0,1,clawZ, 0,0,0,1);
var bRClaw_5_bRPawMatrix = new THREE.Matrix4().multiplyMatrices(bRTorsoMatrix, bRClaw_5_Matrix);

////////////////////////////////
// The Small Middle Tentacles
////////////////////////////////
var rotDegree = Math.PI/3
var x_out = 0.5
var y_out = 0.5
var z_out = noseZ/2

// The bottom ones
var tentacleLeftMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var tentacleRightMatrix = new THREE.Matrix4().set(1,0,0,x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);

var smallTentacleLeftNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, tentacleLeftMatrix);
var smallTentacleRightNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, tentacleRightMatrix);

// The top ones
var rotDegree = -Math.PI/3
var tentacleTopLeftMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var tentacleTopRightMatrix = new THREE.Matrix4().set(1,0,0,x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);

var smallTentacleTopLeftNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, tentacleTopLeftMatrix);
var smallTentacleTopRightNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, tentacleTopRightMatrix);

// Store in an array to loop
var mySmallTentacleMatrix = [tentacleLeftMatrix,tentacleRightMatrix,tentacleTopLeftMatrix,tentacleTopRightMatrix]
var mySmallTentacleNoseMatrix = [smallTentacleLeftNoseMatrix, smallTentacleRightNoseMatrix, smallTentacleTopLeftNoseMatrix, smallTentacleTopRightNoseMatrix]


/////////////////////////////////////////////
// The Middle Large Tentacles
/////////////////////////////////////////////
// The Bottom
var rotDegree = -Math.PI/4
x_out = noseX/3
y_out = noseY/2
z_out = noseZ/1.5

var largeTentacleBottomLeftMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var largeTentacleBottomRightMatrix = new THREE.Matrix4().set(1,0,0,x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);

var largeTentacleBottomLeftNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, largeTentacleBottomLeftMatrix);
var largeTentacleBottomRightNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, largeTentacleBottomRightMatrix);

// The Top
var rotDegree = Math.PI/4
var largeTentacleTopLeftMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var largeTentacleTopRightMatrix = new THREE.Matrix4().set(1,0,0,x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);

var largeTentacleTopLeftNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, largeTentacleTopLeftMatrix);
var largeTentacleTopRightNoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, largeTentacleTopRightMatrix);

var myLargeTentacleMatrix = [largeTentacleBottomLeftMatrix,largeTentacleBottomRightMatrix,largeTentacleTopLeftMatrix,largeTentacleTopRightMatrix]
var myLargeTentacleNoseMatrix = [largeTentacleBottomLeftNoseMatrix,largeTentacleBottomRightNoseMatrix,largeTentacleTopLeftNoseMatrix,largeTentacleTopRightNoseMatrix]

/////////////////////////////////////////////
// The Left Side Large Tentacles
/////////////////////////////////////////////
how_much_rotation = Math.PI/4

// Bottom 2nd from outer
x_out = noseX/10
y_out = noseY*0.9
var rotXDegree = -Math.PI/4
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -how_much_rotation
var lTentLeft_1_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var lTentLeft_1_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, lTentLeft_1_TempMatrix)
var lTentLeft_1_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, lTentLeft_1_Matrix);

// Top 2nd from outer
y_out = noseY*0.9
x_out = -noseX/5
var rotXDegree = 1.2*Math.PI
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -how_much_rotation
var lTentLeft_2_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var lTentLeft_2_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, lTentLeft_2_TempMatrix)
var lTentLeft_2_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, lTentLeft_2_Matrix);

// 3rd from outer
x_out = noseX/10
y_out = noseY
var rotXDegree = -Math.PI/3
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -how_much_rotation
var lTentLeft_3_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var lTentLeft_3_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, lTentLeft_3_TempMatrix)
var lTentLeft_3_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, lTentLeft_3_Matrix);

y_out = noseY*0.9
x_out = -noseX/4.5
z_out = noseZ/1.51
var rotXDegree = 1.275*Math.PI
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -how_much_rotation
var lTentLeft_4_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var lTentLeft_4_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, lTentLeft_4_TempMatrix)
var lTentLeft_4_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, lTentLeft_4_Matrix);

// The two middle ones
x_out = noseX/8
y_out = noseY
z_out = noseZ/1.5
var rotXDegree = -Math.PI/2
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -how_much_rotation
var lTentLeft_5_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var lTentLeft_5_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, lTentLeft_5_TempMatrix)
var lTentLeft_5_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, lTentLeft_5_Matrix);

x_out = -noseX/8
y_out = noseY
var lTentLeft_6_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var lTentLeft_6_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, lTentLeft_6_TempMatrix)
var lTentLeft_6_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, lTentLeft_6_Matrix);

// The middle-most one, has a bigger outward rotation
x_out = 0
y_out = noseY+noseY/16
var rotDegree = -Math.PI/5
// var rotDegree = -Math.PI/2
// var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)
var lTentLeft_7_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var lTentLeft_7_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, lTentLeft_7_TempMatrix)
var lTentLeft_7_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, lTentLeft_7_Matrix);

myLeftLargeTentacleMatrix = [lTentLeft_1_Matrix, lTentLeft_2_Matrix, lTentLeft_3_Matrix, lTentLeft_4_Matrix, lTentLeft_5_Matrix, lTentLeft_6_Matrix, lTentLeft_7_Matrix]
myLeftLargeTentacleNoseMatrix = [lTentLeft_1_NoseMatrix, lTentLeft_2_NoseMatrix, lTentLeft_3_NoseMatrix, lTentLeft_4_NoseMatrix, lTentLeft_5_NoseMatrix, lTentLeft_6_NoseMatrix, lTentLeft_7_NoseMatrix]


/////////////////////////////////////////////
// The Right Side Large Tentacles
/////////////////////////////////////////////
// Bottom 2nd from outer
x_out = -noseX/4.2
y_out = noseY*3/4
var rotXDegree = Math.PI/6
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -Math.PI/4
var rTentLeft_1_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var rTentLeft_1_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, rTentLeft_1_TempMatrix)
var rTentLeft_1_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, rTentLeft_1_Matrix);

// Top 2nd from outer
y_out = -y_out
// x_out = -noseX/3
var rotXDegree = -rotXDegree
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)
var rotDegree = -rotDegree
var rTentLeft_2_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var rTentLeft_2_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, rTentLeft_2_TempMatrix)
var rTentLeft_2_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, rTentLeft_2_Matrix);


// Bottom 2nd from outer
x_out = -noseX/4.2
y_out = noseY*3/3.5
var rotXDegree = Math.PI/4
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -Math.PI/4
var rTentLeft_3_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var rTentLeft_3_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, rTentLeft_3_TempMatrix)
var rTentLeft_3_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, rTentLeft_3_Matrix);

// Top 2nd from outer
y_out = -y_out
// x_out = -noseX/3
var rotXDegree = -rotXDegree
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)
var rotDegree = -rotDegree
var rTentLeft_4_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var rTentLeft_4_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, rTentLeft_4_TempMatrix)
var rTentLeft_4_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, rTentLeft_4_Matrix);


// The two middle ones
x_out = noseX/8
y_out = -noseY
z_out = noseZ/1.5
var rotXDegree = -Math.PI/2
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = Math.PI/4
var rTentLeft_5_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),-y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var rTentLeft_5_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, rTentLeft_5_TempMatrix)
var rTentLeft_5_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, rTentLeft_5_Matrix);

var rotXDegree = Math.PI/2
var rotationXMatrix = new THREE.Matrix4().set(Math.cos(rotXDegree), -Math.sin(rotXDegree), 0, 0, Math.sin(rotXDegree), Math.cos(rotXDegree), 0,0, 0,0,1,0, 0,0,0,1)

var rotDegree = -Math.PI/4
var rTentLeft_6_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var rTentLeft_6_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, rTentLeft_6_TempMatrix)
var rTentLeft_6_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, rTentLeft_6_Matrix);

// The middle most one
y_out = -noseY - noseY/16
x_out = 0
var rotDegree = -Math.PI/4
var rTentLeft_7_TempMatrix = new THREE.Matrix4().set(1,0,0,-x_out, 0,Math.cos(rotDegree),-Math.sin(rotDegree),y_out, 0,Math.sin(rotDegree),Math.cos(rotDegree),z_out, 0,0,0,1);
var rTentLeft_7_Matrix = new THREE.Matrix4().multiplyMatrices(rotationXMatrix, rTentLeft_7_TempMatrix)
var rTentLeft_7_NoseMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadMatrix, rTentLeft_7_Matrix);

myRightLargeTentacleMatrix = [rTentLeft_1_Matrix, rTentLeft_2_Matrix, rTentLeft_3_Matrix, rTentLeft_4_Matrix, rTentLeft_5_Matrix, rTentLeft_6_Matrix, rTentLeft_7_Matrix]
myRightLargeTentacleNoseMatrix = [rTentLeft_1_NoseMatrix, rTentLeft_2_NoseMatrix, rTentLeft_3_NoseMatrix, rTentLeft_4_NoseMatrix, rTentLeft_5_NoseMatrix, rTentLeft_6_NoseMatrix, rTentLeft_7_NoseMatrix]



/////////////////////////////////////////////
// CREATE BODY AND ADD TO SCENE!
/////////////////////////////////////////////
var torso = new THREE.Mesh(torsoGeometry,lambertMaterial);
torso.setMatrix(torsoMatrix)
scene.add(torso);

// TO-DO: PUT TOGETHER THE REST OF YOUR STAR-NOSED MOLE AND ADD TO THE SCENE!
// Hint: Hint: Add one piece of geometry at a time, then implement the motion for that part. 
// Then you can make sure your hierarchy still works properly after each step.
var head = new THREE.Mesh(headGeometry,lambertHeadMaterial);
head.setMatrix(headTorsoMatrix)
scene.add(head);

var nose = new THREE.Mesh(noseGeometry,noseMaterial);
nose.setMatrix(noseHeadMatrix)
scene.add(nose);

var tail = new THREE.Mesh(tailGeometry,pawMaterial);
tail.setMatrix(tailTorsoMatrix)
scene.add(tail);

////////////////////////////////////////////////
// The Front Paws
////////////////////////////////////////////////
var fLPaw = new THREE.Mesh(pawGeometry,pawMaterial);
fLPaw.setMatrix(fLTorsoMatrix)
scene.add(fLPaw);

var fRPaw = new THREE.Mesh(pawGeometry,pawMaterial);
fRPaw.setMatrix(fRTorsoMatrix)
scene.add(fRPaw);

////////////////////////////////////////////////
// The Back Paws
////////////////////////////////////////////////
var bLPaw = new THREE.Mesh(pawGeometry,pawMaterial);
bLPaw.setMatrix(bLTorsoMatrix)
scene.add(bLPaw);

var bRPaw = new THREE.Mesh(pawGeometry,pawMaterial);
bRPaw.setMatrix(bRTorsoMatrix)
scene.add(bRPaw);

/////////////////////////////
// The Front Left Claws
/////////////////////////////
var claw_1_fLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_1_fLPaw.setMatrix(fLClaw_1_fLPawMatrix)
scene.add(claw_1_fLPaw);
var claw_2_fLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_2_fLPaw.setMatrix(fLClaw_2_fLPawMatrix)
scene.add(claw_2_fLPaw);
var claw_3_fLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_3_fLPaw.setMatrix(fLClaw_3_fLPawMatrix)
scene.add(claw_3_fLPaw);
var claw_4_fLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_4_fLPaw.setMatrix(fLClaw_4_fLPawMatrix)
scene.add(claw_4_fLPaw);
var claw_5_fLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_5_fLPaw.setMatrix(fLClaw_5_fLPawMatrix)
scene.add(claw_5_fLPaw);

/////////////////////////////
// The Front Right Claws
/////////////////////////////
var claw_1_fRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_1_fRPaw.setMatrix(fRClaw_1_fRPawMatrix)
scene.add(claw_1_fRPaw);
var claw_2_fRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_2_fRPaw.setMatrix(fRClaw_2_fRPawMatrix)
scene.add(claw_2_fRPaw);
var claw_3_fRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_3_fRPaw.setMatrix(fRClaw_3_fRPawMatrix)
scene.add(claw_3_fRPaw);
var claw_4_fRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_4_fRPaw.setMatrix(fRClaw_4_fRPawMatrix)
scene.add(claw_4_fRPaw);
var claw_5_fRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_5_fRPaw.setMatrix(fRClaw_5_fRPawMatrix)
scene.add(claw_5_fRPaw);

/////////////////////////////
// The Back Left Claws
/////////////////////////////
var claw_1_bLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_1_bLPaw.setMatrix(bLClaw_1_bLPawMatrix)
scene.add(claw_1_bLPaw);
var claw_2_bLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_2_bLPaw.setMatrix(bLClaw_2_bLPawMatrix)
scene.add(claw_2_bLPaw);
var claw_3_bLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_3_bLPaw.setMatrix(bLClaw_3_bLPawMatrix)
scene.add(claw_3_bLPaw);
var claw_4_bLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_4_bLPaw.setMatrix(bLClaw_4_bLPawMatrix)
scene.add(claw_4_bLPaw);
var claw_5_bLPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_5_bLPaw.setMatrix(bLClaw_5_bLPawMatrix)
scene.add(claw_5_bLPaw);

/////////////////////////////
// The Back Right Claws
/////////////////////////////
var claw_1_bRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_1_bRPaw.setMatrix(bRClaw_1_bRPawMatrix)
scene.add(claw_1_bRPaw);
var claw_2_bRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_2_bRPaw.setMatrix(bRClaw_2_bRPawMatrix)
scene.add(claw_2_bRPaw);
var claw_3_bRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_3_bRPaw.setMatrix(bRClaw_3_bRPawMatrix)
scene.add(claw_3_bRPaw);
var claw_4_bRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_4_bRPaw.setMatrix(bRClaw_4_bRPawMatrix)
scene.add(claw_4_bRPaw);
var claw_5_bRPaw = new THREE.Mesh(clawGeometry,clawMaterial);
claw_5_bRPaw.setMatrix(bRClaw_5_bRPawMatrix)
scene.add(claw_5_bRPaw);

// Small tentacles in the middle
var smallTentacle_1 = new THREE.Mesh(smallTentacleGeometry,smallTentacleMaterial);
smallTentacle_1.setMatrix(smallTentacleLeftNoseMatrix)
scene.add(smallTentacle_1);
var smallTentacle_2 = new THREE.Mesh(smallTentacleGeometry,smallTentacleMaterial);
smallTentacle_2.setMatrix(smallTentacleRightNoseMatrix)
scene.add(smallTentacle_2);
var smallTentacle_top_1 = new THREE.Mesh(smallTentacleGeometry,smallTentacleMaterial);
smallTentacle_top_1.setMatrix(smallTentacleTopLeftNoseMatrix)
scene.add(smallTentacle_top_1);
var smallTentacle_top_2 = new THREE.Mesh(smallTentacleGeometry,smallTentacleMaterial);
smallTentacle_top_2.setMatrix(smallTentacleTopRightNoseMatrix)
scene.add(smallTentacle_top_2);
var mySmallTentacles = [smallTentacle_1,smallTentacle_2,smallTentacle_top_1,smallTentacle_top_2]

// Large tentacles in the top and bottom
var largeTentacle_TopLeft = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
largeTentacle_TopLeft.setMatrix(largeTentacleTopLeftNoseMatrix)
scene.add(largeTentacle_TopLeft);
var largeTentacle_TopRight = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
largeTentacle_TopRight.setMatrix(largeTentacleTopRightNoseMatrix)
scene.add(largeTentacle_TopRight);
var largeTentacle_BtmLeft = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
largeTentacle_BtmLeft.setMatrix(largeTentacleBottomLeftNoseMatrix)
scene.add(largeTentacle_BtmLeft);
var largeTentacle_BtmRight = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
largeTentacle_BtmRight.setMatrix(largeTentacleBottomRightNoseMatrix)
scene.add(largeTentacle_BtmRight);
var myLargeTentacles = [largeTentacle_TopLeft,largeTentacle_TopRight,largeTentacle_BtmLeft,largeTentacle_BtmRight]

// Side tentacles on the left
var lTentLeft_1 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
lTentLeft_1.setMatrix(lTentLeft_1_NoseMatrix)
var lTentLeft_2 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
lTentLeft_2.setMatrix(lTentLeft_2_NoseMatrix)
var lTentLeft_3 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
lTentLeft_3.setMatrix(lTentLeft_3_NoseMatrix)
var lTentLeft_4 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
lTentLeft_4.setMatrix(lTentLeft_4_NoseMatrix)
var lTentLeft_5 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
lTentLeft_5.setMatrix(lTentLeft_5_NoseMatrix)
var lTentLeft_6 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
lTentLeft_6.setMatrix(lTentLeft_6_NoseMatrix)
var lTentLeft_7 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
lTentLeft_7.setMatrix(lTentLeft_7_NoseMatrix)
var myLeftLargeTentacles = [lTentLeft_1,lTentLeft_2,lTentLeft_3,lTentLeft_4,lTentLeft_5,lTentLeft_6,lTentLeft_7]
for (var i=0; i < myLeftLargeTentacles.length; i++){
  scene.add(myLeftLargeTentacles[i]);
}

// Side tentacles on the Right
var rTentLeft_1 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
rTentLeft_1.setMatrix(rTentLeft_1_NoseMatrix)
var rTentLeft_2 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
rTentLeft_2.setMatrix(rTentLeft_2_NoseMatrix)
var rTentLeft_3 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
rTentLeft_3.setMatrix(rTentLeft_3_NoseMatrix)
var rTentLeft_4 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
rTentLeft_4.setMatrix(rTentLeft_4_NoseMatrix)
var rTentLeft_5 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
rTentLeft_5.setMatrix(rTentLeft_5_NoseMatrix)
var rTentLeft_6 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
rTentLeft_6.setMatrix(rTentLeft_6_NoseMatrix)
var rTentLeft_7 = new THREE.Mesh(largeTentacleGeometry,tentacleMaterial);
rTentLeft_7.setMatrix(rTentLeft_7_NoseMatrix)

var myRightLargeTentacles = [rTentLeft_1,rTentLeft_2,rTentLeft_3,rTentLeft_4,rTentLeft_5,rTentLeft_6,rTentLeft_7]
for (var i=0; i < myRightLargeTentacles.length; i++){
  scene.add(myRightLargeTentacles[i]);
}

///////////////////////////////////////////////////////////////////////////////////////
//
// ANIMATIONS START HERE!
//
// APPLY DIFFERENT JUMP CUTS/ANIMATIONS TO DIFFERNET KEYS
// Note: The start of "U" animation has been done for you, you must implement the hiearchy and jumpcut.
// Hint: There are other ways to manipulate and grab clock values!!
// Hint: Check THREE.js clock documenation for ideas.
// Hint: It may help to start with a jumpcut and implement the animation after.
// Hint: Where is updateBody() called?
///////////////////////////////////////////////////////////////////////////////////////
var clock = new THREE.Clock(true);

////////////////////////////////
// Torso Global Animation
////////////////////////////////
var p0; // start position or angle
var p1; // end position or angle
var time_length; // total time of animation
var time_start; // start time of animation
var time_end; // end time of animation
var p; // current frame
var animate = false; // animate?
var jumpcut_state = false;
function init_animation(p_start,p_end,t_length){
  p0 = p_start;
  p1 = p_end;
  time_length = t_length;
  time_start = clock.getElapsedTime();
  time_end = time_start + time_length;
  animate = true; // flag for animation
  return;
}

////////////////////////////////
// Head Global Animation
////////////////////////////////
var p0_forward; // start position or angle
var p1_forward; // end position or angle
var p_forward = 0; // current frame
var forward_time_length;
var forward_time_start;
var forward_time_end;
var forward_animate;
function init_forward_animation(p_start,p_end,t_length){
  p0_forward = p_start;
  p1_forward = p_end;
  forward_time_length = t_length;
  forward_time_start = clock.getElapsedTime();
  forward_time_end = forward_time_start + forward_time_length;
  forward_animate = true; // flag for animation
  return;
}

////////////////////////////////
// Head Global Animation
////////////////////////////////
var p0_head; // start position or angle
var p1_head; // end position or angle
var p_head = 0; // current frame
var head_time_length;
var head_time_start;
var head_time_end;
var head_animate;
function init_head_animation(p_start,p_end,t_length){
  p0_head = p_start;
  p1_head = p_end;
  head_time_length = t_length;
  head_time_start = clock.getElapsedTime();
  head_time_end = head_time_start + head_time_length;
  head_animate = true; // flag for animation
  return;
}

////////////////////////////////
// Tail Global Animation
////////////////////////////////
var p0_tail; // start position or angle
var p1_tail; // end position or angle
var p_tail = 0; // current frame
var tail_time_length;
var tail_time_start;
var tail_time_end;
var tail_animate;
function init_tail_animation(p_start,p_end,t_length){
  p0_tail = p_start;
  p1_tail = p_end;
  tail_time_length = t_length;
  tail_time_start = clock.getElapsedTime();
  tail_time_end = tail_time_start + tail_time_length;
  tail_animate = true; // flag for animation
  return;
}

////////////////////////////////
// Fanning Global Animation
////////////////////////////////
var p0_fan; // start position or angle
var p1_fan; // end position or angle
var p_fan = 0; // current frame
var fan_time_length;
var fan_time_start;
var fan_time_end;
var fan_animate = false;
function init_fan_animation(p_start,p_end,t_length){
  p0_fan = p_start;
  p1_fan = p_end;
  fan_time_length = t_length;
  fan_time_start = clock.getElapsedTime();
  fan_time_end = fan_time_start + fan_time_length;
  fan_animate = true; // flag for animation
  return;
}

////////////////////////////////
// Swimming State 1 Paw Animation
////////////////////////////////
var p0_swim_paw_1; // start position or angle
var p1_swim_paw_1; // end position or angle
var p_swim_paw_1 = 0; // current frame
var swim_paw_1_time_length;
var swim_paw_1_time_start;
var swim_paw_1_time_end;
var swim_paw_1_animate = false;
function init_swim_paw_1_animation(p_start,p_end,t_length){
  p0_swim_paw_1 = p_start;
  p1_swim_paw_1 = p_end;
  swim_paw_1_time_length = t_length;
  swim_paw_1_time_start = clock.getElapsedTime();
  swim_paw_1_time_end = swim_paw_1_time_start + swim_paw_1_time_length;
  swim_paw_1_animate = true; // flag for animation
  return;
}

////////////////////////////////
// Swimming State 2 Paw Animation
////////////////////////////////
var p0_swim_paw_2; // start position or angle
var p1_swim_paw_2; // end position or angle
var p_swim_paw_2 = 0; // current frame
var swim_paw_2_time_length;
var swim_paw_2_time_start;
var swim_paw_2_time_end;
var swim_paw_2_animate = false;
function init_swim_paw_2_animation(p_start,p_end,t_length){
  p0_swim_paw_2 = p_start;
  p1_swim_paw_2 = p_end;
  swim_paw_2_time_length = t_length;
  swim_paw_2_time_start = clock.getElapsedTime();
  swim_paw_2_time_end = swim_paw_2_time_start + swim_paw_2_time_length;
  swim_paw_2_animate = true; // flag for animation
  return;
}

////////////////////////////////
// Swimming State 3 Paw Animation
////////////////////////////////
var p0_swim_paw_3; // start position or angle
var p1_swim_paw_3; // end position or angle
var p_swim_paw_3 = 0; // current frame
var swim_paw_3_time_length;
var swim_paw_3_time_start;
var swim_paw_3_time_end;
var swim_paw_3_animate = false;
function init_swim_paw_3_animation(p_start,p_end,t_length){
  p0_swim_paw_3 = p_start;
  p1_swim_paw_3 = p_end;
  swim_paw_3_time_length = t_length;
  swim_paw_3_time_start = clock.getElapsedTime();
  swim_paw_3_time_end = swim_paw_3_time_start + swim_paw_3_time_length;
  swim_paw_3_animate = true; // flag for animation
  return;
}

///////////////////////////////////
// Swimming State 4 Paw Animation
///////////////////////////////////
var p0_swim_paw_4; // start position or angle
var p1_swim_paw_4; // end position or angle
var p_swim_paw_4 = 0; // current frame
var swim_paw_4_time_length;
var swim_paw_4_time_start;
var swim_paw_4_time_end;
var swim_paw_4_animate = false;
function init_swim_paw_4_animation(p_start,p_end,t_length){
  p0_swim_paw_4 = p_start;
  p1_swim_paw_4 = p_end;
  swim_paw_4_time_length = t_length;
  swim_paw_4_time_start = clock.getElapsedTime();
  swim_paw_4_time_end = swim_paw_4_time_start + swim_paw_4_time_length;
  swim_paw_4_animate = true; // flag for animation
  return;
}


////////////////////////////////
// Digging Global Animation
////////////////////////////////
var p0_fDigPaw; // start position or angle
var p1_fDigPaw; // end position or angle
var p_fDigPaw = 0; // current frame
var dig_time_length;
var dig_time_start;
var dig_time_end;
var dig_animate;
function init_dig_animation(p_start,p_end,t_length){
  p0_fDigPaw = p_start;
  p1_fDigPaw = p_end;
  dig_time_length = t_length;
  dig_time_start = clock.getElapsedTime();
  dig_time_end = dig_time_start + dig_time_length;
  dig_animate = true; // flag for animation
  return;
}

////////////////////////////////
// Global Rotation Matrices
////////////////////////////////
var torsoRotMatrix = torsoMatrix;
var headRotMatrix = headMatrix;
var tailRotMatrix = tailMatrix;
var fLPawTorsoRotMatrix = fLTorsoMatrix;
var fRPawTorsoRotMatrix = fRTorsoMatrix;
var bLPawTorsoRotMatrix = bLTorsoMatrix;
var bRPawTorsoRotMatrix = bRTorsoMatrix;
var noseHeadTorsoRotMatrix = noseMatrix;
var tailTorsoRotMatrix;

var fLClaw_1_RotMatrix = fLClaw_1_fLPawMatrix
var fLClaw_2_RotMatrix = fLClaw_2_fLPawMatrix 
var fLClaw_3_RotMatrix = fLClaw_3_fLPawMatrix
var fLClaw_4_RotMatrix = fLClaw_4_fLPawMatrix
var fLClaw_5_RotMatrix = fLClaw_5_fLPawMatrix

var fRClaw_1_RotMatrix = fRClaw_1_fRPawMatrix
var fRClaw_2_RotMatrix = fRClaw_2_fRPawMatrix 
var fRClaw_3_RotMatrix = fRClaw_3_fRPawMatrix
var fRClaw_4_RotMatrix = fRClaw_4_fRPawMatrix
var fRClaw_5_RotMatrix = fRClaw_5_fRPawMatrix

var bLClaw_1_RotMatrix = bLClaw_1_bLPawMatrix
var bLClaw_2_RotMatrix = bLClaw_2_bLPawMatrix 
var bLClaw_3_RotMatrix = bLClaw_3_bLPawMatrix
var bLClaw_4_RotMatrix = bLClaw_4_bLPawMatrix
var bLClaw_5_RotMatrix = bLClaw_5_bLPawMatrix

var bRClaw_1_RotMatrix = bRClaw_1_bRPawMatrix
var bRClaw_2_RotMatrix = bRClaw_2_bRPawMatrix 
var bRClaw_3_RotMatrix = bRClaw_3_bRPawMatrix
var bRClaw_4_RotMatrix = bRClaw_4_bRPawMatrix
var bRClaw_5_RotMatrix = bRClaw_5_bRPawMatrix

var mySmallTentacleRotNoseMatrix = []
var mySmallTentaclePositionMatrix = [];
for (var i=0; i < mySmallTentacles.length; i++){
  mySmallTentacleRotNoseMatrix[i] = mySmallTentacleNoseMatrix[i]
  mySmallTentaclePositionMatrix[i] = mySmallTentacleMatrix[i];
}

var myLargeTentacleRotNoseMatrix = []
var myLargeTentaclePositionMatrix = [];
for (var i=0; i < myLargeTentacles.length; i++){
  myLargeTentacleRotNoseMatrix[i] = myLargeTentacleNoseMatrix[i]
  myLargeTentaclePositionMatrix[i] = myLargeTentacleMatrix[i]
}

var myLeftLargeTentacleRotNoseMatrix = []
var myLeftLargeTentaclePositionMatrix = [];
for (var i=0; i < myLeftLargeTentacleNoseMatrix.length; i++){
  myLeftLargeTentacleRotNoseMatrix[i] = myLeftLargeTentacleNoseMatrix[i]
  myLeftLargeTentaclePositionMatrix[i] = myLeftLargeTentacleMatrix[i]
}

var myRightLargeTentacleRotNoseMatrix = []
var myRightLargeTentaclePositionMatrix = [];
for (var i=0; i < myRightLargeTentacleNoseMatrix.length; i++){
  myRightLargeTentacleRotNoseMatrix[i] = myRightLargeTentacleNoseMatrix[i]
  myRightLargeTentaclePositionMatrix[i] = myRightLargeTentacleMatrix[i]
}

var finalTorsoMatrix = torsoMatrix;
var fLPawTorsoFinalMatrix = frontLeftPawMatrix;
var fRPawTorsoFinalMatrix = frontRightPawMatrix;

////////////////////////////////
// Tilt Body Animation
////////////////////////////////
function anim_tiltBody(rotateZ){
  console.log(torsoMatrix)
  torsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoMatrix, torsoTrackMatrix);
  torsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateZ);
  finalTorsoMatrix = torsoRotMatrix
  torso.setMatrix(torsoRotMatrix);
  
  headTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix,headRotMatrix);
  head.setMatrix(headTorsoRotMatrix);

  noseHeadTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(headTorsoRotMatrix,noseMatrix);
  nose.setMatrix(noseHeadTorsoRotMatrix);

  tailTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix,tailRotMatrix);
  tail.setMatrix(tailTorsoRotMatrix);

  // For the 4 paws
  fLPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, fLPawTorsoFinalMatrix);
  fLPaw.setMatrix(fLPawTorsoRotMatrix);

  // fLPawTorsoRotMatrixTmp = new THREE.Matrix4().multiplyMatrices(frontLeftPawMatrix, torsoRotMatrix);  
  // fLPawTorsoFinalMatrix = fLPawTorsoRotMatrixTmp

  fRPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, fRPawTorsoFinalMatrix);
  fRPaw.setMatrix(fRPawTorsoRotMatrix);  
  // fRPawDigTorsoRotMatrix = fRPawTorsoRotMatrix

  bLPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, backLeftPawMatrix);
  bLPaw.setMatrix(bLPawTorsoRotMatrix);

  bRPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, backRightPawMatrix);
  bRPaw.setMatrix(bRPawTorsoRotMatrix);       

  // For the front left claws
  fLClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_1_Matrix);
  claw_1_fLPaw.setMatrix(fLClaw_1_RotMatrix); 
  fLClawDig_1_RotMatrix = fLClaw_1_RotMatrix  

  fLClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_2_Matrix);
  claw_2_fLPaw.setMatrix(fLClaw_2_RotMatrix);
  fLClawDig_2_RotMatrix = fLClaw_2_RotMatrix   

  fLClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_3_Matrix);
  claw_3_fLPaw.setMatrix(fLClaw_3_RotMatrix); 
  fLClawDig_3_RotMatrix = fLClaw_3_RotMatrix  

  fLClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_4_Matrix);
  claw_4_fLPaw.setMatrix(fLClaw_4_RotMatrix); 
  fLClawDig_4_RotMatrix = fLClaw_4_RotMatrix  

  fLClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_5_Matrix);
  claw_5_fLPaw.setMatrix(fLClaw_5_RotMatrix); 
  fLClawDig_5_RotMatrix = fLClaw_5_RotMatrix  

  // For the front right claws
  fRClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_1_Matrix);
  claw_1_fRPaw.setMatrix(fRClaw_1_RotMatrix);
  fRClawDig_1_RotMatrix = fRClaw_1_RotMatrix  

  fRClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_2_Matrix);
  claw_2_fRPaw.setMatrix(fRClaw_2_RotMatrix); 
  fRClawDig_2_RotMatrix = fRClaw_2_RotMatrix 

  fRClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_3_Matrix);
  claw_3_fRPaw.setMatrix(fRClaw_3_RotMatrix); 
  fRClawDig_3_RotMatrix = fRClaw_3_RotMatrix 

  fRClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_4_Matrix);
  claw_4_fRPaw.setMatrix(fRClaw_4_RotMatrix); 
  fRClawDig_4_RotMatrix = fRClaw_4_RotMatrix 

  fRClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_5_Matrix);
  claw_5_fRPaw.setMatrix(fRClaw_5_RotMatrix); 
  fRClawDig_5_RotMatrix = fRClaw_5_RotMatrix         

  // For the back left claws
  bLClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_1_Matrix);
  claw_1_bLPaw.setMatrix(bLClaw_1_RotMatrix);   

  bLClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_2_Matrix);
  claw_2_bLPaw.setMatrix(bLClaw_2_RotMatrix); 

  bLClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_3_Matrix);
  claw_3_bLPaw.setMatrix(bLClaw_3_RotMatrix); 

  bLClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_4_Matrix);
  claw_4_bLPaw.setMatrix(bLClaw_4_RotMatrix); 

  bLClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_5_Matrix);
  claw_5_bLPaw.setMatrix(bLClaw_5_RotMatrix);       


  // For the back right claws
  bRClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_1_Matrix);
  claw_1_bRPaw.setMatrix(bRClaw_1_RotMatrix);   
  bRClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_2_Matrix);
  claw_2_bRPaw.setMatrix(bRClaw_2_RotMatrix); 
  bRClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_3_Matrix);
  claw_3_bRPaw.setMatrix(bRClaw_3_RotMatrix); 
  bRClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_4_Matrix);
  claw_4_bRPaw.setMatrix(bRClaw_4_RotMatrix); 
  bRClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_5_Matrix);
  claw_5_bRPaw.setMatrix(bRClaw_5_RotMatrix);     

  // Rotate all the tentacles
  for (var i=0; i < mySmallTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,mySmallTentaclePositionMatrix[i]);
    mySmallTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    mySmallTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }

  for (var i=0; i < myLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLargeTentaclePositionMatrix[i]);
    myLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    myLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }                  

  for (var i=0; i < myLeftLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLeftLargeTentaclePositionMatrix[i]);
    myLeftLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    myLeftLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }         

  for (var i=0; i < myRightLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myRightLargeTentaclePositionMatrix[i]);
    myRightLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    myRightLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }          
}

////////////////////////////////
// Forward Moving Animation
////////////////////////////////
function anim_forwardanimate(translateZ){
  torsoRotMatrix2 = new THREE.Matrix4().multiplyMatrices(translateZ, torsoRotMatrix);
  // finalTorsoMatrix = new THREE.Matrix4().multiplyMatrices(translateZ, finalTorsoMatrix);
  torso.setMatrix(torsoRotMatrix2);
  torsoRotMatrix = new THREE.Matrix4().multiplyMatrices(translateZ, torsoRotMatrix);
  torsoTrackMatrix = new THREE.Matrix4().multiplyMatrices(translateZ, torsoTrackMatrix);

  headTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix,headRotMatrix);
  head.setMatrix(headTorsoRotMatrix);

  noseHeadTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(headTorsoRotMatrix,noseMatrix);
  nose.setMatrix(noseHeadTorsoRotMatrix);

  tailTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix,tailRotMatrix);
  tail.setMatrix(tailTorsoRotMatrix);

  // For the 4 paws
  fLPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, fLPawTorsoFinalMatrix);
  fLPaw.setMatrix(fLPawTorsoRotMatrix);

  fRPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, fRPawTorsoFinalMatrix);
  fRPaw.setMatrix(fRPawTorsoRotMatrix);  

  bLPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, backLeftPawMatrix);
  bLPaw.setMatrix(bLPawTorsoRotMatrix);

  bRPawTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, backRightPawMatrix);
  bRPaw.setMatrix(bRPawTorsoRotMatrix);       

  // For the front left claws
  fLClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_1_Matrix);
  claw_1_fLPaw.setMatrix(fLClaw_1_RotMatrix); 
  fLClawDig_1_RotMatrix = fLClaw_1_RotMatrix  

  fLClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_2_Matrix);
  claw_2_fLPaw.setMatrix(fLClaw_2_RotMatrix);
  fLClawDig_2_RotMatrix = fLClaw_2_RotMatrix   

  fLClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_3_Matrix);
  claw_3_fLPaw.setMatrix(fLClaw_3_RotMatrix); 
  fLClawDig_3_RotMatrix = fLClaw_3_RotMatrix  

  fLClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_4_Matrix);
  claw_4_fLPaw.setMatrix(fLClaw_4_RotMatrix); 
  fLClawDig_4_RotMatrix = fLClaw_4_RotMatrix  

  fLClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawTorsoRotMatrix, fLClaw_5_Matrix);
  claw_5_fLPaw.setMatrix(fLClaw_5_RotMatrix); 
  fLClawDig_5_RotMatrix = fLClaw_5_RotMatrix  

  // For the front right claws
  fRClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_1_Matrix);
  claw_1_fRPaw.setMatrix(fRClaw_1_RotMatrix);
  fRClawDig_1_RotMatrix = fRClaw_1_RotMatrix  

  fRClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_2_Matrix);
  claw_2_fRPaw.setMatrix(fRClaw_2_RotMatrix); 
  fRClawDig_2_RotMatrix = fRClaw_2_RotMatrix 

  fRClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_3_Matrix);
  claw_3_fRPaw.setMatrix(fRClaw_3_RotMatrix); 
  fRClawDig_3_RotMatrix = fRClaw_3_RotMatrix 

  fRClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_4_Matrix);
  claw_4_fRPaw.setMatrix(fRClaw_4_RotMatrix); 
  fRClawDig_4_RotMatrix = fRClaw_4_RotMatrix 

  fRClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawTorsoRotMatrix, fRClaw_5_Matrix);
  claw_5_fRPaw.setMatrix(fRClaw_5_RotMatrix); 
  fRClawDig_5_RotMatrix = fRClaw_5_RotMatrix         

  // For the back left claws
  bLClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_1_Matrix);
  claw_1_bLPaw.setMatrix(bLClaw_1_RotMatrix);   

  bLClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_2_Matrix);
  claw_2_bLPaw.setMatrix(bLClaw_2_RotMatrix); 

  bLClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_3_Matrix);
  claw_3_bLPaw.setMatrix(bLClaw_3_RotMatrix); 

  bLClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_4_Matrix);
  claw_4_bLPaw.setMatrix(bLClaw_4_RotMatrix); 

  bLClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawTorsoRotMatrix, bLClaw_5_Matrix);
  claw_5_bLPaw.setMatrix(bLClaw_5_RotMatrix);       


  // For the back right claws
  bRClaw_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_1_Matrix);
  claw_1_bRPaw.setMatrix(bRClaw_1_RotMatrix);   
  bRClaw_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_2_Matrix);
  claw_2_bRPaw.setMatrix(bRClaw_2_RotMatrix); 
  bRClaw_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_3_Matrix);
  claw_3_bRPaw.setMatrix(bRClaw_3_RotMatrix); 
  bRClaw_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_4_Matrix);
  claw_4_bRPaw.setMatrix(bRClaw_4_RotMatrix); 
  bRClaw_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawTorsoRotMatrix, bRClaw_5_Matrix);
  claw_5_bRPaw.setMatrix(bRClaw_5_RotMatrix);     

  // Rotate all the tentacles
  for (var i=0; i < mySmallTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,mySmallTentaclePositionMatrix[i]);
    mySmallTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    mySmallTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }

  for (var i=0; i < myLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLargeTentaclePositionMatrix[i]);
    myLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    myLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }                  

  for (var i=0; i < myLeftLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLeftLargeTentaclePositionMatrix[i]);
    myLeftLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    myLeftLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }         

  for (var i=0; i < myRightLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myRightLargeTentaclePositionMatrix[i]);
    myRightLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrix
    myRightLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }          
}
////////////////////////////////
// Swing Tail Animation
////////////////////////////////
function anim_swingTail(rotateY){
  var tailTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateY);
  tailRotMatrix = new THREE.Matrix4().multiplyMatrices(rotateY, tailMatrix);
  tailRotMatrix2 = new THREE.Matrix4().multiplyMatrices(tailTorsoRotMatrix, tailMatrix);
  tail.setMatrix(tailRotMatrix2);
}

////////////////////////////////
// Swing Head Animation
////////////////////////////////
var headTorsoRotMatrix = torsoRotMatrix;

function anim_swingHead(rotateY){
  headTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateY);
  headRotMatrix = new THREE.Matrix4().multiplyMatrices(rotateY, headMatrix);
  headRotMatrix2 = new THREE.Matrix4().multiplyMatrices(headTorsoRotMatrix, headMatrix);
  head.setMatrix(headRotMatrix2);

  noseHeadTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(headRotMatrix2,noseMatrix);
  nose.setMatrix(noseHeadTorsoRotMatrix);

  // Rotate all the tentacles
  for (var i=0; i < mySmallTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,mySmallTentaclePositionMatrix[i]);
    var tentacleNoseRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,mySmallTentacleMatrix[i]);
    mySmallTentacleRotNoseMatrix[i] = tentacleNoseRotMatrixTemp
    mySmallTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }

  for (var i=0; i < myLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLargeTentaclePositionMatrix[i]);
    var tentacleNoseRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLargeTentacleMatrix[i]);    
    myLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrixTemp
    myLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }     

  for (var i=0; i < myLeftLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLeftLargeTentaclePositionMatrix[i]);
    var tentacleNoseRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myLeftLargeTentacleMatrix[i]);     
    myLeftLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrixTemp
    myLeftLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }         

  for (var i=0; i < myRightLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myRightLargeTentaclePositionMatrix[i]);
    var tentacleNoseRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(noseHeadTorsoRotMatrix,myRightLargeTentacleMatrix[i]);    
    myRightLargeTentacleRotNoseMatrix[i] = tentacleNoseRotMatrixTemp
    myRightLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }         

}

var fLPawDigTorsoRotMatrix = fLPawTorsoRotMatrix;
var fRPawDigTorsoRotMatrix = fRPawTorsoRotMatrix;

var bLPawDigTorsoRotMatrix = bLPawTorsoRotMatrix;
var bRPawDigTorsoRotMatrix = bRPawTorsoRotMatrix;

var fLClawDig_1_RotMatrix = fLClaw_1_RotMatrix;
var fLClawDig_2_RotMatrix = fLClaw_2_RotMatrix;
var fLClawDig_3_RotMatrix = fLClaw_3_RotMatrix;
var fLClawDig_4_RotMatrix = fLClaw_4_RotMatrix;
var fLClawDig_5_RotMatrix = fLClaw_5_RotMatrix;

var bLClawDig_1_RotMatrix = bLClaw_1_RotMatrix;
var bLClawDig_2_RotMatrix = bLClaw_2_RotMatrix;
var bLClawDig_3_RotMatrix = bLClaw_3_RotMatrix;
var bLClawDig_4_RotMatrix = bLClaw_4_RotMatrix;
var bLClawDig_5_RotMatrix = bLClaw_5_RotMatrix;

var fRClawDig_1_RotMatrix = fRClaw_1_RotMatrix;
var fRClawDig_2_RotMatrix = fRClaw_2_RotMatrix;
var fRClawDig_3_RotMatrix = fRClaw_3_RotMatrix;
var fRClawDig_4_RotMatrix = fRClaw_4_RotMatrix;
var fRClawDig_5_RotMatrix = fRClaw_5_RotMatrix;

var bRClawDig_1_RotMatrix = bRClaw_1_RotMatrix;
var bRClawDig_2_RotMatrix = bRClaw_2_RotMatrix;
var bRClawDig_3_RotMatrix = bRClaw_3_RotMatrix;
var bRClawDig_4_RotMatrix = bRClaw_4_RotMatrix;
var bRClawDig_5_RotMatrix = bRClaw_5_RotMatrix;

////////////////////////////////
// Digging Animation
////////////////////////////////
var digTorsoRotMatrix = torsoRotMatrix;

function anim_digPaw(rotateZ_paw, rotateZ_claw){
  digTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateZ_paw);

  fLPawRotMatrix2 = new THREE.Matrix4().multiplyMatrices(digTorsoRotMatrix, frontLeftPawMatrix);
  fLPawTorsoFinalMatrix = new THREE.Matrix4().multiplyMatrices(rotateZ_paw, frontLeftPawMatrix);  
  fLPaw.setMatrix(fLPawRotMatrix2);

  fLClawDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawRotMatrix2,fLClaw_1_Matrix);
  claw_1_fLPaw.setMatrix(fLClawDig_1_RotMatrix);  

  fLClawDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawRotMatrix2,fLClaw_2_Matrix);
  claw_2_fLPaw.setMatrix(fLClawDig_2_RotMatrix);    

  fLClawDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawRotMatrix2,fLClaw_3_Matrix);
  claw_3_fLPaw.setMatrix(fLClawDig_3_RotMatrix);    

  fLClawDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawRotMatrix2,fLClaw_4_Matrix);
  claw_4_fLPaw.setMatrix(fLClawDig_4_RotMatrix);    

  fLClawDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawRotMatrix2,fLClaw_5_Matrix);
  claw_5_fLPaw.setMatrix(fLClawDig_5_RotMatrix);    

  fRPawRotMatrix2 = new THREE.Matrix4().multiplyMatrices(digTorsoRotMatrix, frontRightPawMatrix);
  fRPawTorsoFinalMatrix = new THREE.Matrix4().multiplyMatrices(rotateZ_paw, frontRightPawMatrix);  
  fRPaw.setMatrix(fRPawRotMatrix2);  

  fRClawDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawRotMatrix2,fRClaw_1_Matrix);
  claw_1_fRPaw.setMatrix(fRClawDig_1_RotMatrix);  

  fRClawDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawRotMatrix2,fRClaw_2_Matrix);
  claw_2_fRPaw.setMatrix(fRClawDig_2_RotMatrix);    

  fRClawDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawRotMatrix2,fRClaw_3_Matrix);
  claw_3_fRPaw.setMatrix(fRClawDig_3_RotMatrix);    

  fRClawDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawRotMatrix2,fRClaw_4_Matrix);
  claw_4_fRPaw.setMatrix(fRClawDig_4_RotMatrix);    

  fRClawDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawRotMatrix2,fRClaw_5_Matrix);
  claw_5_fRPaw.setMatrix(fRClawDig_5_RotMatrix);    
}

////////////////////////////////
// Digging Claw Animation
////////////////////////////////

function anim_digClaw(rotateZ_paw, rotateZ_claw){
  var fLClawDigDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLClawDig_1_RotMatrix, rotateZ_claw);
  claw_1_fLPaw.setMatrix(fLClawDigDig_1_RotMatrix);  
  var fLClawDigDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLClawDig_2_RotMatrix, rotateZ_claw);
  claw_2_fLPaw.setMatrix(fLClawDigDig_2_RotMatrix); 
  var fLClawDigDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLClawDig_3_RotMatrix, rotateZ_claw);
  claw_3_fLPaw.setMatrix(fLClawDigDig_3_RotMatrix); 
  var fLClawDigDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLClawDig_4_RotMatrix, rotateZ_claw);
  claw_4_fLPaw.setMatrix(fLClawDigDig_4_RotMatrix); 
  var fLClawDigDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLClawDig_5_RotMatrix, rotateZ_claw);
  claw_5_fLPaw.setMatrix(fLClawDigDig_5_RotMatrix);   
  var fRClawDigDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRClawDig_1_RotMatrix, rotateZ_claw);
  claw_1_fRPaw.setMatrix(fRClawDigDig_1_RotMatrix);   
  var fRClawDigDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRClawDig_2_RotMatrix, rotateZ_claw);
  claw_2_fRPaw.setMatrix(fRClawDigDig_2_RotMatrix); 
  var fRClawDigDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRClawDig_3_RotMatrix, rotateZ_claw);
  claw_3_fRPaw.setMatrix(fRClawDigDig_3_RotMatrix); 
  var fRClawDigDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRClawDig_4_RotMatrix, rotateZ_claw);
  claw_4_fRPaw.setMatrix(fRClawDigDig_4_RotMatrix); 
  var fRClawDigDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRClawDig_5_RotMatrix, rotateZ_claw);
  claw_5_fRPaw.setMatrix(fRClawDigDig_5_RotMatrix);
}

/////////////////////////////////////////
// Swimming Paw Animation 1: Front Left
/////////////////////////////////////////
function anim_swimPaws_1(rotateZ_paw){
  fLPawDigTorsoRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateZ_paw);
  fLPawDigTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawDigTorsoRotMatrixTemp, frontLeftPawMatrix);
  fLPaw.setMatrix(fLPawDigTorsoRotMatrix);

  fLClawDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawDigTorsoRotMatrix,fLClaw_1_Matrix);
  claw_1_fLPaw.setMatrix(fLClawDig_1_RotMatrix);   
  fLClawDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawDigTorsoRotMatrix,fLClaw_2_Matrix);
  claw_2_fLPaw.setMatrix(fLClawDig_2_RotMatrix); 
  fLClawDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawDigTorsoRotMatrix,fLClaw_3_Matrix);
  claw_3_fLPaw.setMatrix(fLClawDig_3_RotMatrix); 
  fLClawDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawDigTorsoRotMatrix,fLClaw_4_Matrix);
  claw_4_fLPaw.setMatrix(fLClawDig_4_RotMatrix); 
  fLClawDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fLPawDigTorsoRotMatrix,fLClaw_5_Matrix);
  claw_5_fLPaw.setMatrix(fLClawDig_5_RotMatrix);   
}

////////////////////////////////
// Swimming Paw Animation 2: Front Right
////////////////////////////////
function anim_swimPaws_2(rotateZ_paw){
  fRPawDigTorsoRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateZ_paw);
  fRPawDigTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawDigTorsoRotMatrixTemp, frontRightPawMatrix);
  fRPaw.setMatrix(fRPawDigTorsoRotMatrix);

  fRClawDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawDigTorsoRotMatrix,fRClaw_1_Matrix);
  claw_1_fRPaw.setMatrix(fRClawDig_1_RotMatrix);   
  fRClawDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawDigTorsoRotMatrix,fRClaw_2_Matrix);
  claw_2_fRPaw.setMatrix(fRClawDig_2_RotMatrix); 
  fRClawDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawDigTorsoRotMatrix,fRClaw_3_Matrix);
  claw_3_fRPaw.setMatrix(fRClawDig_3_RotMatrix); 
  fRClawDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawDigTorsoRotMatrix,fRClaw_4_Matrix);
  claw_4_fRPaw.setMatrix(fRClawDig_4_RotMatrix); 
  fRClawDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(fRPawDigTorsoRotMatrix,fRClaw_5_Matrix);
  claw_5_fRPaw.setMatrix(fRClawDig_5_RotMatrix);   
}

////////////////////////////////
// Swimming Paw Animation 3: Back Left
////////////////////////////////
function anim_swimPaws_3(rotateZ_paw){
  bLPawDigTorsoRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateZ_paw);
  bLPawDigTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawDigTorsoRotMatrixTemp, backLeftPawMatrix);
  bLPaw.setMatrix(bLPawDigTorsoRotMatrix);

  bLClawDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawDigTorsoRotMatrix,bLClaw_1_Matrix);
  claw_1_bLPaw.setMatrix(bLClawDig_1_RotMatrix);   
  bLClawDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawDigTorsoRotMatrix,bLClaw_2_Matrix);
  claw_2_bLPaw.setMatrix(bLClawDig_2_RotMatrix); 
  bLClawDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawDigTorsoRotMatrix,bLClaw_3_Matrix);
  claw_3_bLPaw.setMatrix(bLClawDig_3_RotMatrix); 
  bLClawDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawDigTorsoRotMatrix,bLClaw_4_Matrix);
  claw_4_bLPaw.setMatrix(bLClawDig_4_RotMatrix); 
  bLClawDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(bLPawDigTorsoRotMatrix,bLClaw_5_Matrix);
  claw_5_bLPaw.setMatrix(bLClawDig_5_RotMatrix);    
}

/////////////////////////////////////////
// Swimming Paw Animation 4: Back Right
/////////////////////////////////////////
function anim_swimPaws_4(rotateZ_paw){
  bRPawDigTorsoRotMatrixTemp = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, rotateZ_paw);
  bRPawDigTorsoRotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawDigTorsoRotMatrixTemp, backRightPawMatrix);
  bRPaw.setMatrix(bRPawDigTorsoRotMatrix);

  bRClawDig_1_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawDigTorsoRotMatrix,bRClaw_1_Matrix);
  claw_1_bRPaw.setMatrix(bRClawDig_1_RotMatrix);   
  bRClawDig_2_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawDigTorsoRotMatrix,bRClaw_2_Matrix);
  claw_2_bRPaw.setMatrix(bRClawDig_2_RotMatrix); 
  bRClawDig_3_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawDigTorsoRotMatrix,bRClaw_3_Matrix);
  claw_3_bRPaw.setMatrix(bRClawDig_3_RotMatrix); 
  bRClawDig_4_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawDigTorsoRotMatrix,bRClaw_4_Matrix);
  claw_4_bRPaw.setMatrix(bRClawDig_4_RotMatrix); 
  bRClawDig_5_RotMatrix = new THREE.Matrix4().multiplyMatrices(bRPawDigTorsoRotMatrix,bRClaw_5_Matrix);
  claw_5_bRPaw.setMatrix(bRClawDig_5_RotMatrix);  
}

////////////////////////////////
// Fanning Animation
////////////////////////////////
function anim_fanout(rotateZ_fan, rotateZ_fan_reverse){
  console.log("fan animation called")
  for (var i=0; i < 2; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(mySmallTentacleRotNoseMatrix[i], rotateZ_fan);
    mySmallTentaclePositionMatrix[i] = new THREE.Matrix4().multiplyMatrices(mySmallTentacleMatrix[i], rotateZ_fan)
    mySmallTentacles[i].setMatrix(tentacleNoseRotMatrix)

    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myLargeTentacleRotNoseMatrix[i], rotateZ_fan_reverse);
    myLargeTentaclePositionMatrix[i] = new THREE.Matrix4().multiplyMatrices(myLargeTentacleMatrix[i], rotateZ_fan_reverse)
    myLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }

  for (var i=2; i < 4; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(mySmallTentacleRotNoseMatrix[i],rotateZ_fan_reverse);
    mySmallTentaclePositionMatrix[i] = new THREE.Matrix4().multiplyMatrices(mySmallTentacleMatrix[i], rotateZ_fan_reverse)    
    mySmallTentacles[i].setMatrix(tentacleNoseRotMatrix)

    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myLargeTentacleRotNoseMatrix[i],rotateZ_fan);
    myLargeTentaclePositionMatrix[i] = new THREE.Matrix4().multiplyMatrices(myLargeTentacleMatrix[i], rotateZ_fan)
    myLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)    
  }

  for (var i=0; i < myLeftLargeTentacles.length; i++){
    var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myLeftLargeTentacleRotNoseMatrix[i],rotateZ_fan_reverse);
    myLeftLargeTentaclePositionMatrix[i] = new THREE.Matrix4().multiplyMatrices(myLeftLargeTentacleMatrix[i], rotateZ_fan_reverse)   
    myLeftLargeTentacles[i].setMatrix(tentacleNoseRotMatrix)
  }        

  var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleRotNoseMatrix[1],rotateZ_fan);
  myRightLargeTentaclePositionMatrix[1] = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleMatrix[1],rotateZ_fan);
  myRightLargeTentacles[1].setMatrix(tentacleNoseRotMatrix)

  var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleRotNoseMatrix[0],rotateZ_fan_reverse);
  myRightLargeTentaclePositionMatrix[0] = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleMatrix[0],rotateZ_fan_reverse);  
  myRightLargeTentacles[0].setMatrix(tentacleNoseRotMatrix)

  var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleRotNoseMatrix[2],rotateZ_fan_reverse);
  myRightLargeTentaclePositionMatrix[2] = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleMatrix[2],rotateZ_fan_reverse);  
  myRightLargeTentacles[2].setMatrix(tentacleNoseRotMatrix)

  var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleRotNoseMatrix[3],rotateZ_fan);
  myRightLargeTentaclePositionMatrix[3] = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleMatrix[3],rotateZ_fan);    
  myRightLargeTentacles[3].setMatrix(tentacleNoseRotMatrix)

  var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleRotNoseMatrix[4],rotateZ_fan);
  myRightLargeTentaclePositionMatrix[4] = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleMatrix[4],rotateZ_fan);      
  myRightLargeTentacles[4].setMatrix(tentacleNoseRotMatrix)
  
  var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleRotNoseMatrix[5],rotateZ_fan_reverse);
  myRightLargeTentaclePositionMatrix[5] = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleMatrix[5],rotateZ_fan_reverse);  
  myRightLargeTentacles[5].setMatrix(tentacleNoseRotMatrix)      
  
  var tentacleNoseRotMatrix = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleRotNoseMatrix[6],rotateZ_fan_reverse);
  myRightLargeTentaclePositionMatrix[6] = new THREE.Matrix4().multiplyMatrices(myRightLargeTentacleMatrix[6],rotateZ_fan_reverse);    
  myRightLargeTentacles[6].setMatrix(tentacleNoseRotMatrix)       

}
//////////////////////////////////////////////////////
// Global animation states
//////////////////////////////////////////////////////
var p_fDigClaw;
var p_fDigClaw_end;
var p_fDigClaw_start;
var animate_claw_flag = false;
var claw_time_end;
var claw_time_start;
var hasRotatedTorso = false;
var pawRotAngle;
var fireball = new THREE.Mesh(fireballGeometry,fireballMaterial);
var fireballCurrentMatrix = fireballMatrix;
var hasPlacedFireball = false;
var arrayOfFireballs = []
var arrayOfFireballMatrix = []

var move_animate = false;
var hasStartMoving = false;

/////////////////////////////////////////////
// Main Update Body Loop
/////////////////////////////////////////////
function updateBody() {

  /////////////////////////////////////////////
  // For animating the torso
  /////////////////////////////////////////////
  if(animate){
    if(!jumpcut_state){
      var time = clock.getElapsedTime(); // t seconds passed since the clock started.
      if (time > time_end){
        p = p1;
        animate = false;
      }
      p = (p1 - p0)*((time-time_start)/time_length) + p0; // current frame 
    }
    else{
      p = p1
      animate = false;
    }
    var rotateZ = new THREE.Matrix4().set(1,        0,         0,        0, 
                                          0, Math.cos(-p),-Math.sin(-p), 0, 
                                          0, Math.sin(-p), Math.cos(-p), 0,
                                          0,        0,         0,        1);
    anim_tiltBody(rotateZ);
  }
  if(forward_animate){
    var translateZ = new THREE.Matrix4().set(1,0,0,0, 
                                          0, 1,0, 0, 
                                          0, 0, 1, 0.1,
                                          0,0,0,1);
    anim_forwardanimate(translateZ);
  }  
  if(walk_animate){
    var walk_time = clock.getElapsedTime();
    if (walk_time > walk_time_start_master+10){
      walk_animate = false
      forward_animate = false
      swim_paw_1_animate = false
      swim_paw_2_animate = false
      swim_paw_3_animate = false
      swim_paw_4_animate = false
      tail_animate = false
      head_animate = false
      sit_state = 0
    }
    if (walk_time > walk_time_start + 0.2){
      console.log("walk")
      pawRotAngle = (-1)*pawRotAngle; // flip angle
      init_swim_paw_1_animation(p_swim_paw_1,p_swim_paw_1+pawRotAngle*2,0.5)
      init_swim_paw_2_animation(p_swim_paw_2,p_swim_paw_2+(-pawRotAngle*2),0.5)
      init_swim_paw_3_animation(p_swim_paw_3,p_swim_paw_3-pawRotAngle*2,0.5)
      init_swim_paw_4_animation(p_swim_paw_4,p_swim_paw_4+(pawRotAngle*2),0.5)      
      init_tail_animation(p_tail, p_tail+pawRotAngle,0.5)
      init_forward_animation(0,Math.PI/16,5)
      init_head_animation(p_head, p_head+pawRotAngle, 0.8)    
      walk_time_start = clock.getElapsedTime(); // Reset Clock 
    }
  }

  ///////////////////////////
  // For fight anim
  ///////////////////////////
  if(fight_animate){
    if(!hasRotatedTorso){
      init_animation(0,Math.PI/3,0.8)
      init_swim_paw_1_animation(0,-Math.PI/4,1)
      init_swim_paw_2_animation(0,Math.PI/4,1)   
      pawRotAngle = -Math.PI/4;
      console.log(pawRotAngle)
      p_head = 0
      p_tail = 0
      hasRotatedTorso = true
    }

    var fight_time = clock.getElapsedTime();
    if (fight_time > fight_time_start + 0.5 && hasPlacedFireball == false){
      console.log(fireballGeometry)
      console.log(fireballMaterial)
      fireball.setMatrix(fireballMatrix)
      scene.add(fireball);
      arrayOfFireballs.push(fireball)
      arrayOfFireballMatrix.push(fireballMatrix)
      hasPlacedFireball = true;
    }

    if (fight_time > fight_time_start + 0.2){
      console.log("Fight")
      pawRotAngle = (-1)*pawRotAngle; // flip angle
      init_swim_paw_1_animation(p_swim_paw_1,p_swim_paw_1+pawRotAngle*2,0.5)
      init_swim_paw_2_animation(p_swim_paw_2,p_swim_paw_2+(-pawRotAngle*2),0.5)
      init_tail_animation(p_tail, p_tail+pawRotAngle,0.5)
      init_head_animation(p_head, p_head+pawRotAngle, 0.8)
      for (var i=0; i < arrayOfFireballs.length; i++){
        if (Math.random() > 0.5){
          var sign = -1
        }
        else{
          var sign = 1
        }
        var fireballMovementMatrix = new THREE.Matrix4().set(1,0,0,sign*(0.1+Math.random()), 0,1,0,0.1+Math.random(), 0,0,1,1+sign*Math.random(), 0,0,0,1);        
        var fireballMatrixTemp = new THREE.Matrix4().multiplyMatrices(fireballMovementMatrix, arrayOfFireballMatrix[i]);
        arrayOfFireballMatrix[i] = fireballMatrixTemp
        arrayOfFireballs[i].setMatrix(fireballMatrixTemp)
        scene.add(arrayOfFireballs[i])
      }
      
      if (Math.random() > 0.2){
        var fireballGeometry = makeCube();
        var r = Math.random()
        var non_uniform_scale = new THREE.Matrix4().set(1+2*r,0,0,0, 0,1+2*r,0,0, 0,0,1+2*r,0, 0,0,0,1);

        // var non_uniform_scale = new THREE.Matrix4().set(1+Math.random()*2.0,0,0,0, 0,2+Math.random()*2.5,0,0, 0,0,1+Math.random()*1.5,0, 0,0,0,1);
        fireballGeometry.applyMatrix(non_uniform_scale);        
        var fireballMaterial = new THREE.MeshLambertMaterial({
          color: orange,
          shininess: 5
        });
      
        var fireballMatrix_temp = new THREE.Matrix4().set(1,0,0,-2+Math.random(), 0,1,0,1+Math.random()*2, 0,0,1,12+Math.random()*2, 0,0,0,1);
        var fireballMatrix_new = new THREE.Matrix4().multiplyMatrices(torsoRotMatrix, fireballMatrix_temp)
        var fireball_new = new THREE.Mesh(fireballGeometry,fireballMaterial);

        arrayOfFireballs.push(fireball_new)
        if(Math.random() < 0.3){
          arrayOfFireballs[arrayOfFireballs.length-1].material.color.setRGB(1.0,0.667,0)
        }
        else if(Math.random() >= 0.5){
          arrayOfFireballs[arrayOfFireballs.length-1].material.color.setRGB(0.60+Math.random(),0.02,0.04);
        }
        else{
          arrayOfFireballs[arrayOfFireballs.length-1].material.color.setRGB(0.34+Math.random(),0,0);
        }
        arrayOfFireballMatrix.push(fireballMatrix_new)  
        console.log(arrayOfFireballs)
      }
      fight_time_start = clock.getElapsedTime(); // Reset Clock 
    }
  }

  ////////////////////////////////////
  // For animating the head
  ////////////////////////////////////
  if(head_animate){
    if(!jumpcut_state){
      var head_time = clock.getElapsedTime(); // t seconds passed since the clock started.

      if (head_time > head_time_end){
        p_head = p1_head;
        head_animate = false;
      }

      p_head = (p1_head - p0_head)*((head_time-head_time_start)/head_time_length) + p0_head; // current frame 
    }
    else{
      p_head = p1_head
      head_animate = false;
    }

    var rotateY = new THREE.Matrix4().set(Math.cos(-p_head),  0, Math.sin(-p_head), 0, 
                                          0,            1,    0,        0, 
                                          -Math.sin(-p_head), 0, Math.cos(-p_head), 0,
                                          0,            0,    0,        1);
    anim_swingHead(rotateY);
  }

  ////////////////////////////////////
  // For fanning animation
  ////////////////////////////////////
  if(fan_animate){
    if(!jumpcut_state){
      var fan_time = clock.getElapsedTime(); // t seconds passed since the clock started.

      if (fan_time > fan_time_end){
        p_fan = p1_fan;
        fan_animate = false;
      }

      p_fan = (p1_fan - p0_fan)*((fan_time-fan_time_start)/fan_time_length) + p0_fan; // current frame 
    }
    else{
      p_fan = p1_fan
      fan_animate = false;
    }

    var rotateZ = new THREE.Matrix4().set(1,        0,         0,      0, 
                                          0, Math.cos(-p_fan),-Math.sin(-p_fan), 0, 
                                          0, Math.sin(-p_fan), Math.cos(-p_fan), 0,
                                          0,        0,         0,      1);
    var rotateZ_reversed = new THREE.Matrix4().set(1,        0,         0,      0, 
                                          0, Math.cos(p_fan),-Math.sin(p_fan), 0, 
                                          0, Math.sin(p_fan), Math.cos(p_fan), 0,
                                          0,        0,         0,      1);      
    anim_fanout(rotateZ, rotateZ_reversed);
  }  

  ///////////////////////////
  // For tail animation
  ///////////////////////////
  if(tail_animate){
    if(!jumpcut_state){
      var tail_time = clock.getElapsedTime(); // t seconds passed since the clock started.

      if (tail_time > tail_time_end){
        p_tail = p1_tail;
        tail_animate = false;
      }

      p_tail = (p1_tail - p0_tail)*((tail_time-tail_time_start)/tail_time_length) + p0_tail; // current frame 
    }
    else{
      p_tail = p1_tail
      tail_animate = false;
    }

    var rotateY = new THREE.Matrix4().set(Math.cos(p_tail),  0, Math.sin(p_tail), 0, 
                                          0,            1,    0,        0, 
                                          -Math.sin(p_tail), 0, Math.cos(p_tail), 0,
                                          0,            0,    0,        1);
    anim_swingTail(rotateY);
  }

  /////////////////////////////////////////////
  // For swim state 1 paw animation
  /////////////////////////////////////////////
  if(swim_paw_1_animate){
    if(!jumpcut_state){
      var swim_paw_1_time = clock.getElapsedTime(); // t seconds passed since the clock started.

      if (swim_paw_1_time > swim_paw_1_time_end){
        p_swim_paw_1 = p1_swim_paw_1;
        swim_paw_1_animate = false;
      }

      p_swim_paw_1 = (p1_swim_paw_1 - p0_swim_paw_1)*((swim_paw_1_time-swim_paw_1_time_start)/swim_paw_1_time_length) + p0_swim_paw_1; // current frame 
    }
    else{
      p_swim_paw_1 = p1_swim_paw_1
      swim_paw_1_animate = false;
    }

    var rotateZ_paw = new THREE.Matrix4().set(1,        0,         0,      0, 
                                          0, Math.cos(-p_swim_paw_1),-Math.sin(-p_swim_paw_1), 0, 
                                          0, Math.sin(-p_swim_paw_1), Math.cos(-p_swim_paw_1), 0,
                                          0,        0,         0,      1);

    anim_swimPaws_1(rotateZ_paw);
  }  

  /////////////////////////////////////////////
  // For swim state 2 paw animation
  /////////////////////////////////////////////
  if(swim_paw_2_animate){
    if(!jumpcut_state){
      var swim_paw_2_time = clock.getElapsedTime(); // t seconds passed since the clock started.

      if (swim_paw_2_time > swim_paw_2_time_end){
        p_swim_paw_2 = p1_swim_paw_2;
        swim_paw_2_animate = false;
      }

      p_swim_paw_2 = (p1_swim_paw_2 - p0_swim_paw_2)*((swim_paw_2_time-swim_paw_2_time_start)/swim_paw_2_time_length) + p0_swim_paw_2; // current frame 
    }
    else{
      p_swim_paw_2 = p1_swim_paw_2
      swim_paw_2_animate = false;
    }

    var rotateZ_paw = new THREE.Matrix4().set(1,        0,         0,      0, 
                                          0, Math.cos(-p_swim_paw_2),-Math.sin(-p_swim_paw_2), 0, 
                                          0, Math.sin(-p_swim_paw_2), Math.cos(-p_swim_paw_2), 0,
                                          0,        0,         0,      1);

    anim_swimPaws_2(rotateZ_paw);
  }    
  /////////////////////////////////////////////
  // For swim state 3 paw animation
  /////////////////////////////////////////////
  if(swim_paw_3_animate){
    if(!jumpcut_state){
      var swim_paw_3_time = clock.getElapsedTime(); // t seconds passed since the clock started.

      if (swim_paw_3_time > swim_paw_3_time_end){
        p_swim_paw_3 = p1_swim_paw_3;
        swim_paw_3_animate = false;
      }

      p_swim_paw_3 = (p1_swim_paw_3 - p0_swim_paw_3)*((swim_paw_3_time-swim_paw_3_time_start)/swim_paw_3_time_length) + p0_swim_paw_3; // current frame 
    }
    else{
      p_swim_paw_3 = p1_swim_paw_3
      swim_paw_3_animate = false;
    }

    var rotateZ_paw = new THREE.Matrix4().set(1,        0,         0,      0, 
                                          0, Math.cos(-p_swim_paw_3),-Math.sin(-p_swim_paw_3), 0, 
                                          0, Math.sin(-p_swim_paw_3), Math.cos(-p_swim_paw_3), 0,
                                          0,        0,         0,      1);

    anim_swimPaws_3(rotateZ_paw);
  }      

  /////////////////////////////////////////////
  // For swim state 4 paw animation
  /////////////////////////////////////////////
  if(swim_paw_4_animate){
    if(!jumpcut_state){
      var swim_paw_4_time = clock.getElapsedTime(); // t seconds passed since the clock started.

      if (swim_paw_4_time > swim_paw_4_time_end){
        p_swim_paw_4 = p1_swim_paw_4;
        swim_paw_4_animate = false;
      }

      p_swim_paw_4 = (p1_swim_paw_4 - p0_swim_paw_4)*((swim_paw_4_time-swim_paw_4_time_start)/swim_paw_4_time_length) + p0_swim_paw_4; // current frame 
    }
    else{
      p_swim_paw_4 = p1_swim_paw_4
      swim_paw_4_animate = false;
    }

    var rotateZ_paw = new THREE.Matrix4().set(1,        0,         0,      0, 
                                          0, Math.cos(-p_swim_paw_4),-Math.sin(-p_swim_paw_4), 0, 
                                          0, Math.sin(-p_swim_paw_4), Math.cos(-p_swim_paw_4), 0,
                                          0,        0,         0,      1);

    anim_swimPaws_4(rotateZ_paw);
  }

  ////////////////////////////////////
  // For digging animation
  ////////////////////////////////////
  if(key != "D"){
    dig_animate = false
    animate_claw_flag = false
    dig_state = "rest"
  }
  if(dig_animate==true){
    if (animate_claw_flag == true && dig_state == "digging"){
      var dig_time = clock.getElapsedTime();

      if (dig_time > claw_time_end){
        console.log('stopping claw rotation')
        animate_claw_flag = false;
        dig_animate = false;
      }

      console.log('rotating claws')
      var dig_time = clock.getElapsedTime();
      var rotateZ_claw = new THREE.Matrix4().set(1,        0,         0,      0, 
                                        0, Math.cos(-p_fDigClaw),-Math.sin(-p_fDigClaw), 0, 
                                        0, Math.sin(-p_fDigClaw), Math.cos(-p_fDigClaw), 0,
                                        0,        0,         0,      1);          
      anim_digClaw(rotateZ_paw, rotateZ_claw);
      p_fDigClaw = (p_fDigClaw_end - p_fDigClaw_start)*((dig_time-dig_time_start)/dig_time_length) + p_fDigClaw_start;
    }

    else{
      if(!jumpcut_state){
        var dig_time = clock.getElapsedTime(); // t seconds passed since the clock started.
        console.log(time)
        if (dig_time > dig_time_end){
          animate_claw_flag = true
          var claw_time_length = 0.4;
          claw_time_start = clock.getElapsedTime();
          claw_time_end = claw_time_start + claw_time_length;            
          console.log('fdigpaw: ', p_fDigPaw)
          if (dig_state == "rest"){
            animate_claw_flag = false;
            dig_animate = false;
          }
        }

        p_fDigPaw = (p1_fDigPaw - p0_fDigPaw)*((dig_time-dig_time_start)/dig_time_length) + p0_fDigPaw; // current frame 
        p_fDigClaw_start = p_fDigPaw
      }
      else{
        p_fDigPaw = p1_fDigPaw
        p_fDigClaw_start = p_fDigPaw
        dig_animate = false;
      }
      
      var rotateZ_paw = new THREE.Matrix4().set(1,    0,         0,      0, 
                                            0, Math.cos(-p_fDigPaw),-Math.sin(-p_fDigPaw), 0, 
                                            0, Math.sin(-p_fDigPaw), Math.cos(-p_fDigPaw), 0,
                                            0,        0,         0,      1);

      anim_digPaw(rotateZ_paw, rotateZ_claw);
    }

  }

}

/////////////////////////////////////////////////////////////////////////////////
// LISTEN TO KEYBOARD
// Hint: Pay careful attention to how the keys already specified work!
/////////////////////////////////////////////////////////////////////////////////
var keyboard = new THREEx.KeyboardState();
var grid_state = false;
var key;
var body_state = 0;
var body_initial = 0;
var p = 0;

var head_state = "rest";
var head_initial = 0;
var p_head = 0;

var dig_state = "rest";
var dig_initial = 0;
var p_fDigPaw = 0;
var swim_anim_state = 0;

var fan_state = 0;
var tail_state = 0;

var sit_state = 0;
var fight_animate = false;
var fight_time_start;
var fight_time_interval;
var walk_animate = false;
var walk_time_start;
var walk_time_start_master;
var walk_time_interval;

var move_state = 0;

keyboard.domElement.addEventListener('keydown',function(event){
  if (event.repeat)
    return;
  if(keyboard.eventMatches(event,"Z")){  // Z: Reveal/Hide helper grid
    grid_state = !grid_state;
    grid_state? scene.add(grid) : scene.remove(grid);}   
  else if(keyboard.eventMatches(event,"0")){    // 0: Set camera to neutral position, view reset
    camera.position.set(45,0,0);
    camera.lookAt(scene.position);}
  else if(keyboard.eventMatches(event,"U")){ 
    console.log(body_state)
    if (body_state == 0) {
      init_animation(p,Math.PI/4,1)
      console.log(body_state)
      body_state++
    }
    else if (body_state == 1) {
      init_animation(p,0,1)
      console.log(body_state)
      body_state = 0;
    }    
    else if (body_state == -1) {
      init_animation(p,0,1)
      console.log(body_state)
      body_state = 0;
    }     
  }  
  else if(keyboard.eventMatches(event,"E")){ 
    console.log(body_state)
    if (body_state == 0) {
      init_animation(p,-Math.PI/4,1)
      console.log(body_state)
      body_state--
    }
    else if (body_state == -1) {
      init_animation(p,0,1)
      console.log(body_state)
      body_state = 0;
    }    
    else if (body_state == 1) {
      init_animation(p,0,1)
      console.log(body_state)
      body_state = 0;
    }     
  }  
  else if(keyboard.eventMatches(event,"V")){ 
    if(tail_state == 0){
      init_tail_animation(0,Math.PI/8,1)
      tail_state--;
    }
    else if(tail_state == 1){
      init_tail_animation(p1_tail,p0_tail,1)
      tail_state = 0;      
    }
    else if(tail_state == -1){
      init_tail_animation(p_tail,0,1)
      tail_state = 0;
    }
  }
  else if(keyboard.eventMatches(event,"T")){ 
    if (tail_state == 0){
      init_tail_animation(p_tail,-Math.PI/8,1)
      tail_state++;
    }
    else if(tail_state == -1){
      init_tail_animation(p_tail,0,1)
      tail_state = 0;      
    }
    else if(tail_state == 1){
      init_tail_animation(p_tail,0,1)
      tail_state = 0;
    }
  }
  else if(keyboard.eventMatches(event,"N")){ 
    if (fan_state == 0){
      init_fan_animation(0,Math.PI/8,0.4)
      fan_state++;
    }
    else{
      init_fan_animation(Math.PI/8,0,0.4)
      fan_state = 0;
    }
  }  
  else if(keyboard.eventMatches(event,"G")){
    console.log(head_state) 
    if (key == "G" && head_state != "rest") {
      head_state = "rest";
      init_head_animation(p_head,head_initial,0.5)
      console.log(head_state)
    }
    else if(key == "H" && head_state != "rest"){
      head_state = "rest";
      init_head_animation(p_head,0,0.5)
      console.log(head_state)
    }  
    else{
      head_state = "left";
      init_head_animation(p_head,Math.PI/8,0.5);
      console.log(head_state)
    }
    key = "G"
  }
  else if(keyboard.eventMatches(event,"X")){
    console.log(sit_state)
    if(sit_state == 0){
      fight_animate = true
      fight_time_start = clock.getElapsedTime();
      fight_time_interval = 2
      sit_state++;
    }
    else if(sit_state == 1){
      fight_animate = false    
      hasPlacedFireball = false;
      hasRotatedTorso = false;      
      for(var i=0; i<arrayOfFireballs.length; i++){
        scene.remove(arrayOfFireballs[i])
      }      
      arrayOfFireballs = []
      arrayOfFireballMatrix = [] 
      init_animation(p,0,0.2)     
      init_swim_paw_1_animation(p_swim_paw_1,Math.PI/8,0.5)
      init_swim_paw_2_animation(p_swim_paw_2,Math.PI/8,0.5)
      init_swim_paw_3_animation(p_swim_paw_3,-Math.PI/8,0.5)
      init_swim_paw_4_animation(p_swim_paw_4,-Math.PI/8,0.5)
      init_tail_animation(p_tail, 0,0.5)
      init_head_animation(p_head, 0,0.8)   
      sit_state++
    } 
    else if(sit_state == 2){         
      p_swim_paw_1 = 0
      p_swim_paw_2 = 0
      p_swim_paw_3 = 0
      p_swim_paw_4 = 0
      walk_animate = true
      walk_time_start = clock.getElapsedTime();  
      walk_time_start_master = clock.getElapsedTime();
      sit_state++
    }
    else if(sit_state == 3){
      console.log(torsoTrackMatrix)
      walk_animate = false
      forward_animate = false
      // torsoMatrix = torsoTrackMatrix
      // torsoTrackMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1);
      init_swim_paw_1_animation(p_swim_paw_1,0,0.5)
      init_swim_paw_2_animation(p_swim_paw_2,0,0.5)
      init_swim_paw_3_animation(p_swim_paw_3,0,0.5)
      init_swim_paw_4_animation(p_swim_paw_4,0,0.5)
      init_tail_animation(p_tail,0,1)
      init_head_animation(p_head,0,1)      
      sit_state = 0
    }
  }
  else if(keyboard.eventMatches(event,"S")){
    console.log(swim_anim_state)
    if(swim_anim_state == 0){
      // Front left paw out and back right paw out
      init_swim_paw_1_animation(0,-Math.PI/8,1)
      init_swim_paw_4_animation(0,-Math.PI/8,1)

      // Tail swing to left
      init_tail_animation(0,Math.PI/8,1)

      // Fan out
      init_fan_animation(0,Math.PI/8,1)

      // Head to the right
      init_head_animation(0,-Math.PI/8,1)

      swim_anim_state++
    }
    else if(swim_anim_state == 1){
      // Front left paw retract and back right paw retract
      init_swim_paw_1_animation(p_swim_paw_1,0,1)
      init_swim_paw_4_animation(p_swim_paw_1,0,1)

      // Front right paw out and back left paw out
      init_swim_paw_2_animation(p_swim_paw_2,-Math.PI/8,1)
      init_swim_paw_3_animation(p_swim_paw_2,-Math.PI/8,1)

      // Tail swing to right
      init_tail_animation(p_tail,-Math.PI/8,1)

      // Head to the left
      init_head_animation(p_head,Math.PI/8,1)

      swim_anim_state++
    }
    else{
      // Front right paw retract and back left paw retract
      init_swim_paw_2_animation(p_swim_paw_2,0,1)
      init_swim_paw_3_animation(p_swim_paw_2,0,1)

      // Tail swing to the middle
      init_tail_animation(p_tail,0,1)

      // Head to the middle
      init_head_animation(p_head,0,1)    

      // Fan in
      init_fan_animation(Math.PI/8,0,1)

      swim_anim_state = 0
    }
  }
  else if(keyboard.eventMatches(event,"H")){
    console.log(head_state) 
    if (key == "H" && head_state != "rest") {
      console.log(p_head)
      head_state = "rest";
      init_head_animation(p_head,head_initial,0.5)
      console.log(head_state)
    }
    else if(key == "G" && head_state != "rest"){
      head_state = "rest";
      init_head_animation(p_head,0,0.5)
      console.log(head_state)
    }  
    else{
      head_state = "right";
      init_head_animation(p_head,-Math.PI/8,0.5);
      console.log(head_state)
    }
    key = "H"
  }
  else if(keyboard.eventMatches(event,"D")){
    console.log(dig_state) 
    if (key == "D" && dig_state != "rest") {
      dig_state = "rest";
      init_dig_animation(p_fDigPaw,0,0.2)
      p_fDigClaw_end = 0
      p_fDigClaw_start = 0
      console.log(dig_state)
      console.log(p_fDigPaw)
    }
    else{
      dig_state = "digging";
      init_dig_animation(p_fDigPaw,-Math.PI/8,0.2);
      p_fDigClaw_end = p_fDigPaw-Math.PI/8-Math.PI/32
      p_fDigClaw_start = p_fDigPaw-Math.PI/8      
      console.log(dig_state)
      console.log(p_fDigPaw)
      key = "D"
    }
  }  

  else if(keyboard.eventMatches(event,"space")){ 
    jumpcut_state = !jumpcut_state;}

});

// SETUP UPDATE CALL-BACK
// Hint: It is useful to understand what is being updated here, the effect, and why.
function update() {

  updateBody();

  requestAnimationFrame(update);
  renderer.render(scene,camera);
}

update();